@@ -37,16 +37,17 @@ class RiakError(Exception) :
                 return repr(self.value)
 
 from riak_object import RiakObject
-from bucket import RiakBucket 
+from bucket import RiakBucket
 from client import RiakClient
-from mapreduce import RiakMapReduce, RiakMapReducePhase, RiakLinkPhase
-from transports.pbc import RiakPbcTransport
-from transports.http import RiakHttpTransport
+from mapreduce import RiakMapReduce, RiakMapReducePhase, RiakLinkPhase,\
+    RiakKeyFilter
+from transports.pbc import RiakPbcTransport, RiakPbcCachedTransport
+from transports.http import RiakHttpTransport, RiakHttpReuseTransport, RiakHttpPoolTransport
 
 ONE = "one"
 ALL = "all"
 QUORUM = "quorum"
 
-
+key_filter = RiakKeyFilter()
 
 
