@@ -8,12 +8,16 @@ except ImportError:
     import simplejson as json
 import os
 import random
-import sys
 import unittest
+import uuid
+import time
+
 from riak import RiakClient
-from riak import RiakPbcTransport
-from riak import RiakHttpTransport
+from riak import RiakPbcTransport, RiakPbcCachedTransport
+from riak import RiakHttpTransport, RiakHttpPoolTransport, RiakHttpReuseTransport
 from riak import RiakKeyFilter, key_filter
+from riak.mapreduce import RiakLink
+from riak.test_server import TestServer
 
 HOST = os.environ.get('RIAK_TEST_HOST', 'localhost')
 HTTP_HOST = os.environ.get('RIAK_TEST_HTTP_HOST', HOST)
@@ -21,6 +25,17 @@ PB_HOST = os.environ.get('RIAK_TEST_PB_HOST', HOST)
 HTTP_PORT = int(os.environ.get('RIAK_TEST_HTTP_PORT', '8098'))
 PB_PORT = int(os.environ.get('RIAK_TEST_PB_PORT', '8087'))
 SKIP_SEARCH = int(os.environ.get('SKIP_SEARCH', '0'))
+USE_TEST_SERVER = int(os.environ.get('USE_TEST_SERVER', '0'))
+
+if USE_TEST_SERVER:
+    HTTP_PORT = 9000
+    PB_PORT = 9002
+    test_server = TestServer()
+    test_server.cleanup()
+    test_server.prepare()
+    test_server.start()
+
+SKIP_LUWAK = int(os.environ.get('SKIP_LUWAK', '0'))
 
 class NotJsonSerializable(object):
 
@@ -446,7 +461,6 @@ class BaseTestCase(object):
         self.assertEqual(len(results), 3)
 
     def test_store_binary_object_from_file(self):
-        print __file__
         bucket = self.client.bucket('bucket')
         rand = str(self.randint())
         obj = bucket.new_binary_from_file('foo_from_file', os.path.dirname(__file__) + "/test_all.py")
@@ -470,7 +484,6 @@ class BaseTestCase(object):
         obj.set_usermeta({'custom': 'some metadata'})
         obj.store()
         obj = bucket.get('fooster')
-        print(obj.get_usermeta())
         self.assertEqual('some metadata', obj.get_usermeta()['custom'])
 
     def test_store_binary_object_from_file_should_fail_if_file_not_found(self):
@@ -694,6 +707,23 @@ class RiakPbcTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn,
                             client_id = zero_client_id)
         self.assertEqual(zero_client_id, c.get_client_id()) #
 
+class RiakPbcCachedTransportCase(BaseTestCase, MapReduceAliasTestMixIn,
+                               unittest.TestCase):
+    def setUp(self):
+        self.host = PB_HOST
+        self.port = PB_PORT
+        self.transport_class = RiakPbcCachedTransport
+        super(RiakPbcCachedTransportCase, self).setUp()
+
+    def test_uses_client_id_if_given(self):
+        self.host = PB_HOST
+        self.port = PB_PORT
+        zero_client_id = "\0\0\0\0"
+        c = RiakClient(PB_HOST, PB_PORT,
+                            transport_class = RiakPbcCachedTransport,
+                            client_id = zero_client_id)
+        self.assertEqual(zero_client_id, c.get_client_id()) #
+
 
 class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.TestCase):
 
@@ -717,6 +747,193 @@ class RiakHttpTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.
         bucket.new(None, data={}).store()
         self.assertEqual(len(bucket.get_keys()), 1)
 
+    def test_too_many_link_headers_shouldnt_break_http(self):
+        bucket = self.client.bucket("bucket")
+        o = bucket.new("lots_of_links", "My god, it's full of links!")
+        for i in range(0, 400):
+            link = RiakLink("other", "key%d" % i, "next")
+            o.add_link(link)
+
+        o.store()
+        stored_object = bucket.get("lots_of_links")
+        self.assertEqual(len(stored_object.get_links()), 400)
+
+    def test_bucket_search_enabled(self):
+        bucket = self.client.bucket("unsearch_bucket")
+        self.assertFalse(bucket.search_enabled())
+
+    def test_enable_search_commit_hook(self):
+        bucket = self.client.bucket("search_bucket")
+        bucket.enable_search()
+        self.assertTrue(self.client.bucket("search_bucket").search_enabled())
+
+    def test_disable_search_commit_hook(self):
+        bucket = self.client.bucket("no_search_bucket")
+        bucket.enable_search()
+        self.assertTrue(self.client.bucket("no_search_bucket").search_enabled())
+        bucket.disable_search()
+        self.assertFalse(self.client.bucket("no_search_bucket").search_enabled())
+
+    def test_store_file_with_luwak(self):
+        if SKIP_LUWAK:
+            return True
+
+        file = os.path.dirname(__file__) + "/test_all.py"
+        with open(file, "r") as input_file:
+            data = input_file.read()
+
+        key = uuid.uuid1().hex
+        self.client.store_file(key, data)
+
+    def test_store_get_file_with_luwak(self):
+        if SKIP_LUWAK:
+            return True
+
+        file = os.path.dirname(__file__) + "/test_all.py"
+        with open(file, "r") as input_file:
+            data = input_file.read()
+
+        key = uuid.uuid1().hex
+        self.client.store_file(key, data)
+        time.sleep(1)
+        file = self.client.get_file(key)
+        self.assertEquals(data, file)
+
+    def test_delete_file_with_luwak(self):
+        if SKIP_LUWAK:
+            return True
+
+        file = os.path.dirname(__file__) + "/test_all.py"
+        with open(file, "r") as input_file:
+            data = input_file.read()
+
+        key = uuid.uuid1().hex
+        self.client.store_file(key, data)
+        time.sleep(1)
+        self.client.delete_file(key)
+        time.sleep(1)
+        file = self.client.get_file(key)
+        self.assertIsNone(file)
+
+    def test_solr_search_from_bucket(self):
+        if SKIP_SEARCH:
+            return True
+        bucket = self.client.bucket('searchbucket')
+        bucket.new("user", {"username": "roidrage"}).store()
+        results = bucket.search("username:roidrage")
+        self.assertEquals(1, len(results["response"]["docs"]))
+
+    def test_solr_search_with_params_from_bucket(self):
+        if SKIP_SEARCH:
+            return True
+        bucket = self.client.bucket('searchbucket')
+        bucket.new("user", {"username": "roidrage"}).store()
+        results = bucket.search("username:roidrage", wt="xml")
+        self.assertEquals(1, len(list(results.find("result").iter("doc"))))
+
+    def test_solr_search_with_params(self):
+        if SKIP_SEARCH:
+            return True
+        bucket = self.client.bucket('searchbucket')
+        bucket.new("user", {"username": "roidrage"}).store()
+        results = self.client.solr().search("searchbucket", "username:roidrage", wt="xml")
+        self.assertEquals(1, len(list(results.find("result").iter("doc"))))
+
+    def test_solr_search(self):
+        if SKIP_SEARCH:
+            return True
+        bucket = self.client.bucket('searchbucket')
+        bucket.new("user", {"username": "roidrage"}).store()
+        results = self.client.solr().search("searchbucket", "username:roidrage")
+        self.assertEquals(1, len(results["response"]["docs"]))
+
+    def test_add_document_to_index(self):
+        if SKIP_SEARCH:
+            return True
+
+        self.client.solr().add("searchbucket", {"id": "doc", "username": "tony"})
+        results = self.client.solr().search("searchbucket", "username:tony")
+        self.assertEquals("tony", results["response"]["docs"][0]["fields"]["username"])
+
+    def test_add_multiple_documents_to_index(self):
+        if SKIP_SEARCH:
+            return True
+        self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
+        results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
+        self.assertEquals(2, len(results["response"]["docs"]))
+
+    def test_delete_documents_from_search_by_id(self):
+        if SKIP_SEARCH:
+            return True
+        self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
+        self.client.solr().delete("searchbucket", docs=["dizzy"])
+        results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
+        self.assertEquals(1, len(results["response"]["docs"]))
+
+    def test_delete_documents_from_search_by_query(self):
+        if SKIP_SEARCH:
+            return True
+        self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
+        self.client.solr().delete("searchbucket", queries=["username:dizzy", "username:russell"])
+        results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
+        self.assertEquals(0, len(results["response"]["docs"]))
+
+    def test_delete_documents_from_search_by_query_and_id(self):
+        if SKIP_SEARCH:
+            return True
+        self.client.solr().add("searchbucket", {"id": "dizzy", "username": "dizzy"}, {"id": "russell", "username": "russell"})
+        self.client.solr().delete("searchbucket", docs=["dizzy"], queries=["username:russell"])
+        results = self.client.solr().search("searchbucket", "username:russell OR username:dizzy")
+        self.assertEquals(0, len(results["response"]["docs"]))
+
+class RiakHttpPoolTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.TestCase):
+
+    def setUp(self):
+        self.host = HTTP_HOST
+        self.port = HTTP_PORT
+        self.transport_class = RiakHttpPoolTransport
+        super(RiakHttpPoolTransportTestCase, self).setUp()
+
+    def test_no_returnbody(self):
+        bucket = self.client.bucket("bucket")
+        o = bucket.new("foo", "bar").store(return_body=False)
+        self.assertEqual(o.vclock(), None)
+
+    def test_generate_key(self):
+        # Ensure that Riak generates a random key when
+        # the key passed to bucket.new() is None.
+        bucket = self.client.bucket('random_key_bucket')
+        for key in bucket.get_keys():
+            bucket.get(str(key)).delete()
+        bucket.new(None, data={}).store()
+        self.assertEqual(len(bucket.get_keys()), 1)
+
+    def test_set_client_id(self):
+        self.client.set_client_id("Client")
+        self.assertEqual(self.client.get_transport().get_client_id(), "Client")
+
+class RiakHttpReuseTransportTestCase(BaseTestCase, MapReduceAliasTestMixIn, unittest.TestCase):
+
+    def setUp(self):
+        self.host = HTTP_HOST
+        self.port = HTTP_PORT
+        self.transport_class = RiakHttpReuseTransport
+        super(RiakHttpReuseTransportTestCase, self).setUp()
+
+    def test_no_returnbody(self):
+        bucket = self.client.bucket("bucket")
+        o = bucket.new("foo", "bar").store(return_body=False)
+        self.assertEqual(o.vclock(), None)
+
+    def test_generate_key(self):
+        # Ensure that Riak generates a random key when
+        # the key passed to bucket.new() is None.
+        bucket = self.client.bucket('random_key_bucket')
+        for key in bucket.get_keys():
+            bucket.get(str(key)).delete()
+        bucket.new(None, data={}).store()
+        self.assertEqual(len(bucket.get_keys()), 1)
+
 
 class RiakTestFilter(unittest.TestCase):
     def test_simple(self):
