@@ -31,17 +31,17 @@ class RiakTransport(object):
 
     @classmethod
     def make_random_client_id(self):
-        '''
+        """
         Returns a random client identifier
-        '''
+        """
         return 'py_%s' % base64.b64encode(
                 str(random.randint(1, 1073741824)))
 
     @classmethod
     def make_fixed_client_id(self):
-        '''
+        """
         Returns a unique identifier for the current machine/process/thread.
-        '''
+        """
         machine = platform.node()
         process = os.getpid()
         thread = threading.currentThread().getName()
@@ -106,5 +106,37 @@ class RiakTransport(object):
         raise RiakError("not implemented")
 
     def set_client_id(self, client_id):
+        """
+        Set the client id. This overrides the default, random client id, which is automatically
+        generated when none is specified in when creating the transport object.
+        """
         raise RiakError("not implemented")
 
+    def get_client_id(self):
+        """
+        Fetch the client id for the transport.
+        """
+        raise RiakError("not implemented")
+
+    def store_file(self, key, content_type="application/octet-stream", content=None):
+        """
+        Store a large piece of data in luwak.
+        key = the key/filename for the object
+        content_type = the object's content type
+        content = the object's data
+        """
+        raise RiakError("luwak not supported by this transport.")
+
+    def get_file(self, key):
+        """
+        Get an object from luwak.
+        key = the object's key
+        """
+        raise RiakError("luwak not supported by this transport.")
+
+        """
+        Delete an object in luwak.
+        key = the object's key
+        """
+    def delete_file(self, key):
+        raise RiakError("luwak not supported by this transport.")
